#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from django.conf import settings
from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import render_to_response
from django.http import HttpResponseBadRequest, HttpResponseNotFound, HttpResponse
from .models import PaynovaPayment
from .signals import paynova_payment

import hashlib
import logging
import sys

log = logging.getLogger(__name__)
_ver = sys.version_info


@csrf_exempt
def paynova_success(request):
    return render_to_response('paynova/success.html', {'data': request.POST})


@csrf_exempt
def paynova_cancel(request):
    return render_to_response('paynova/cancel.html', {'data': request.POST})


@csrf_exempt
def paynova_pending(request):
    return render_to_response('paynova/pending.html', {'data': request.POST})


@csrf_exempt
def paynova_callback(request):
    """
        Handle Event Hook Notifications from Paynova
    """

    # check DIGEST

    if not _ehn_checksum(request.POST):
        log.error('EHN DIGEST hash is not verified. %s' % request.POST)
        return HttpResponseBadRequest()

    # check EVENT_TYPE

    if request.POST.get('EVENT_TYPE') != 'PAYMENT':
        log.error('Unexpected EVENT_TYPE. %s' % request.POST)
        return HttpResponseBadRequest()

    # get PaynovaPayment from model

    try:
        pp = PaynovaPayment.objects.get(order_id=request.POST.get('ORDER_ID'), session_id=request.POST.get('SESSION_ID'))
    except PaynovaPayment.DoesNotExist:
        log.error('Unknown ORDER_ID. %s' % request.POST)
        return HttpResponseNotFound()

    pp.status = request.POST.get('PAYMENT_STATUS')
    pp.status_reason = request.POST.get('PAYMENT_STATUS_REASON')
    pp.params_ehn = request.POST
    pp.save()

    # send signal

    paynova_payment.send(sender=pp, params=request.POST, status=request.POST.get('PAYMENT_STATUS'))

    return HttpResponse()


def _ehn_checksum(data):
    hash_string = '%s;%s;%s;%s;%s;' % (
        data.get('EVENT_TYPE'),
        data.get('EVENT_TIMESTAMP'),
        data.get('DELIVERY_TIMESTAMP'),
        data.get('MERCHANT_ID'),
        settings.PAYNOVA_SECRET
    )

    if _ver >= (3, 0):
        hash_string = hash_string.encode('ascii')

    h = hashlib.sha1()
    h.update(hash_string)

    return data.get('DIGEST') == h.hexdigest()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from django.db import models

import json


class PaynovaPayment(models.Model):
    """
        Model for storing all transactions to Paynova
        *status* shows current status of payment.
    """

    STATUS_ORDER_CREATED = 'ORDER_CREATED'
    STATUS_PAYMENT_INITED = 'PAYMENT_INITED'
    STATUS_CANCELLED = 'CANCELLED'
    STATUS_DECLINED = 'DECLINED'
    STATUS_ERROR = 'ERROR'
    STATUS_PENDING = 'PENDING'
    STATUS_COMPLETED = 'COMPLETED'
    STATUS_AUTHORIZED = 'AUTHORIZED'

    date = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=255, null=True, blank=True)
    status_reason = models.TextField(null=True, blank=True)

    order_id = models.CharField(max_length=255, null=True, blank=True, unique=True)
    session_id = models.CharField(max_length=255, null=True, blank=True, unique=True)
    url = models.CharField(max_length=255, null=True, blank=True)

    _params_create_order = models.TextField(null=True, blank=True)
    _params_init_payment = models.TextField(null=True, blank=True)
    _params_ehn = models.TextField(null=True, blank=True)

    @property
    def params_create_order(self):
        return json.loads(self._params_create_order)

    @params_create_order.setter
    def params_create_order(self, value):
        self._params_create_order = json.dumps(value)

    @property
    def params_init_payment(self):
        return json.loads(self._params_init_payment)

    @params_init_payment.setter
    def params_init_payment(self, value):
        self._params_init_payment = json.dumps(value)

    @property
    def params_ehn(self):
        return json.loads(self._params_ehn)

    @params_ehn.setter
    def params_ehn(self, value):
        self._params_ehn = json.dumps(value)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

import os

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "settings")

from django_nose import NoseTestSuiteRunner


if __name__ == '__main__':
    NoseTestSuiteRunner(verbosity=1).run_tests(['tests'])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from preggy import expect
from tests.base import TestCase
from httmock import HTTMock
from django.test.utils import override_settings
from django_paynova import create_order, initialize_payment
from django_paynova.models import PaynovaPayment
from django_paynova.payment import _get_url_params, _get_params_for_initialize_payment
from paynova_api_python_client import PaynovaException
from .mock import paynova_mock


class PaymentTestCase(TestCase):
    def setUp(self):
        PaynovaPayment.objects.all().delete()

    def tearDown(self):
        pass

    def test_create_order_success(self):
        with HTTMock(paynova_mock):
            pp = create_order(self.create_order_params, init_payment=False)
            expect(pp).to_be_instance_of(PaynovaPayment)
            expect(pp.order_id).to_equal(TestCase.ORDER_ID)
            expect(pp.status).to_equal(PaynovaPayment.STATUS_ORDER_CREATED)
            expect(pp.params_create_order).to_equal(self.create_order_params)

    def test_create_order_fail(self):
        with HTTMock(paynova_mock):
            params = self.create_order_params.copy()
            params['orderNumber'] = '0002'

            with expect.error_to_happen(PaynovaException):
                create_order(params, init_payment=False)

            pp = PaynovaPayment.objects.filter().first()
            expect(pp).to_be_instance_of(PaynovaPayment)
            expect(pp.order_id).to_be_null()
            expect(pp.status).to_equal(PaynovaPayment.STATUS_ERROR)
            expect(pp.status_reason).not_to_be_null()
            expect(pp.params_create_order).to_equal(params)

    def test_get_url_params(self):
        url_params = _get_url_params()
        expect(url_params).to_length(4)

    @override_settings(PAYNOVA_CALLBACK_URL=None)
    def test_get_url_params_fail(self):
        with expect.error_to_happen(PaynovaException):
            _get_url_params()

    def test_get_params_for_initialize_payment(self):
        with expect.error_to_happen(PaynovaException):
            _get_params_for_initialize_payment({})

        with expect.error_to_happen(PaynovaException):
            _get_params_for_initialize_payment({'orderId': 'not-exists'})

        with HTTMock(paynova_mock):
            pp = create_order(self.create_order_params, init_payment=False)
            params, pp = _get_params_for_initialize_payment({
                'orderId': pp.order_id,
                'test': 'test'
            })
            expect(pp).to_be_instance_of(PaynovaPayment)
            expect(params).to_be_instance_of(dict)
            expect(params.get('test')).to_equal('test')
            expect(params.get('orderId')).to_equal(pp.order_id)
            expect(params.get('totalAmount')).to_equal(pp.params_create_order.get('totalAmount'))
            expect(params).to_include('interfaceOptions')
            expect(params['interfaceOptions']).to_length(6)

    def test_initialize_payment_success(self):
        with HTTMock(paynova_mock):
            pp = create_order(self.create_order_params, init_payment=False)
            pp = initialize_payment({'orderId': pp.order_id})

            expect(pp).to_be_instance_of(PaynovaPayment)
            expect(pp.order_id).to_equal(TestCase.ORDER_ID)
            expect(pp.session_id).to_equal(TestCase.ORDER_SESSION_ID)
            expect(pp.url).to_equal(TestCase.ORDER_URL)
            expect(pp.status).to_equal(PaynovaPayment.STATUS_PAYMENT_INITED)
            expect(pp.params_create_order).to_equal(self.create_order_params)
            expect(pp.params_init_payment).not_to_be_null()

    def test_initialize_payment_fail(self):
        with HTTMock(paynova_mock):
            pp = create_order(self.create_order_params, init_payment=False)

            with expect.error_to_happen(PaynovaException):
                initialize_payment({'orderId': pp.order_id, 'should_fail': True})

            pp = PaynovaPayment.objects.filter().first()
            expect(pp).to_be_instance_of(PaynovaPayment)
            expect(pp.order_id).to_equal(TestCase.ORDER_ID)
            expect(pp.session_id).to_be_null()
            expect(pp.url).to_be_null()
            expect(pp.status).to_equal(PaynovaPayment.STATUS_ERROR)
            expect(pp.status_reason).not_to_be_null()
            expect(pp.params_create_order).to_equal(self.create_order_params)
            expect(pp.params_init_payment).not_to_be_null()

    def test_create_and_init(self):
        with HTTMock(paynova_mock):
            pp = create_order(self.create_order_params, init_payment=True)
            expect(pp).to_be_instance_of(PaynovaPayment)
            expect(pp.order_id).to_equal(TestCase.ORDER_ID)
            expect(pp.session_id).to_equal(TestCase.ORDER_SESSION_ID)
            expect(pp.url).to_equal(TestCase.ORDER_URL)
            expect(pp.status).to_equal(PaynovaPayment.STATUS_PAYMENT_INITED)
            expect(pp.params_create_order).to_equal(self.create_order_params)
            expect(pp.params_init_payment).not_to_be_null()


from tests.base import TestCase
from httmock import all_requests

import json

@all_requests
def paynova_mock(url, request):
    def success(content=None):
        if not content:
            content = {}

        content['status'] = {
            'isSuccess': True,
            'errorNumber': 0,
            'statusKey': 'SUCCESS',
        }

        return {'status_code': 200, 'content': content}

    def fail(content=None, errorNumber=None, statusKey=None):
        if not content:
            content = {}

        content['status'] = {
            'isSuccess': False,
            'errorNumber': errorNumber,
            'statusKey': statusKey,
        }

        return {'status_code': 200, 'content': content}

    if url[2] == '/api/orders/create':
        params = json.loads(request.body)
        if params.get('orderNumber') == '0001':
            return success({
                'orderId': TestCase.ORDER_ID
            })
        else:
            return fail()

    elif url[2] == ('/api/orders/%s/initializePayment' % TestCase.ORDER_ID):
        params = json.loads(request.body)
        if params.get('should_fail'):
            return fail()
        else:
            return success({
                'sessionId': TestCase.ORDER_SESSION_ID,
                'url': TestCase.ORDER_URL
            })

    return {'status_code': 404}


DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3'
    }
}

INSTALLED_APPS = (
    'django_paynova',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

SECRET_KEY = 'any-key'

ROOT_URLCONF = 'urls'

PAYNOVA_MERCHANT_ID = 'MERCHANT_ID'
PAYNOVA_PASSWORD = 'PASSWORD'
PAYNOVA_SECRET = 'SECRET'
PAYNOVA_CALLBACK_URL = 'http://mysite.com'


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of django-paynova.
# https://github.com/akolpakov/django-paynova

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Andrey Kolpakov <aakolpakov@gmail.com>

from django.test import TestCase


class TestCase(TestCase):
    ORDER_ID = '70bf60e7-cc9b-4321-bb32-a449010f45a5'
    ORDER_SESSION_ID = 'test-session-id'
    ORDER_URL = 'http://test.com'

    create_order_params = {
        'orderNumber': '0001',
        'currencyCode': 'EUR',
        'totalAmount': 10
    }